"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformMixin = void 0;
const tslib_1 = require("tslib");
const typeorm_1 = require("@nestjs/typeorm");
const common_1 = require("@nestjs/common");
const param_case_1 = require("param-case");
const constants_1 = require("../../constants");
const mixin_1 = require("../../helpers/mixin");
function transformMixin(entity, connectionName) {
    let MixinTransform = class MixinTransform {
        transformData(data) {
            const { metadata } = this.getRepository(data.constructor.name);
            const relations = this.getRelations(metadata);
            const relationships = relations.reduce((accum, relation) => {
                accum[relation] = {};
                const builtLinks = {
                    self: this.getRelationshipLink(param_case_1.paramCase(metadata.name), data.id, relation),
                    related: this.getDirectLink(param_case_1.paramCase(metadata.name), data.id, relation),
                };
                if (data[relation] !== undefined) {
                    let builtData;
                    if (Array.isArray(data[relation])) {
                        builtData = data[relation].map(relationData => ({
                            type: param_case_1.paramCase(this.getRepository(relationData.constructor.name).metadata.name),
                            id: relationData.id.toString()
                        }));
                    }
                    else if (data[relation] !== null) {
                        builtData = {
                            type: param_case_1.paramCase(this.getRepository(data[relation].constructor.name).metadata.name),
                            id: data[relation].id.toString()
                        };
                    }
                    else {
                        builtData = null;
                    }
                    accum[relation].data = builtData;
                }
                accum[relation].links = builtLinks;
                return accum;
            }, {});
            const attributes = Object
                .entries(data)
                .filter(([key, val]) => {
                return key !== 'id' && !relations.find(rel => rel === key);
            })
                .reduce((accum, [key, val]) => {
                accum[key] = val;
                return accum;
            }, {});
            const result = {
                id: data.id.toString(),
                type: param_case_1.paramCase(metadata.name),
                attributes,
                relationships,
                links: {
                    self: this.getResourceLink(param_case_1.paramCase(metadata.name), data.id.toString()),
                }
            };
            if (Object.keys(relationships).length === 0) {
                delete result.relationships;
            }
            return result;
        }
        transformInclude(data) {
            const { metadata } = this.getRepository(data.constructor.name);
            const relations = this.getRelations(metadata);
            return Object
                .entries(data)
                .filter(([key, value]) => {
                return relations.find(relation => relation === key) && !!value;
            })
                .reduce((accum, [key, val]) => {
                accum.push(...(Array.isArray(val) ? val : [val]));
                return accum;
            }, [])
                .map(value => this.transformData(value));
        }
        getRelationshipLink(resourceName, resourceId, relationName) {
            const prefix = this.config.globalPrefix ? `/${this.config.globalPrefix}/` : '/';
            return `${prefix}${resourceName}/${resourceId}/relationships/${relationName}`;
        }
        getResourceLink(resourceName, resourceId) {
            const prefix = this.config.globalPrefix ? `/${this.config.globalPrefix}/` : '/';
            return `${prefix}${resourceName}/${resourceId}`;
        }
        getDirectLink(resourceName, resourceId, relationName) {
            const prefix = this.config.globalPrefix ? `/${this.config.globalPrefix}/` : '/';
            return `${prefix}${resourceName}/${resourceId}/${relationName}`;
        }
        getRepository(name) {
            return this.repository.manager.getRepository(name);
        }
        getRelations(metadata) {
            return metadata.relations.map(relation => relation.propertyPath);
        }
    };
    tslib_1.__decorate([
        typeorm_1.InjectRepository(entity, connectionName),
        tslib_1.__metadata("design:type", Object)
    ], MixinTransform.prototype, "repository", void 0);
    tslib_1.__decorate([
        common_1.Inject(constants_1.JSON_API_CONFIG),
        tslib_1.__metadata("design:type", Object)
    ], MixinTransform.prototype, "config", void 0);
    MixinTransform = tslib_1.__decorate([
        common_1.Injectable()
    ], MixinTransform);
    return mixin_1.mixin(MixinTransform);
}
exports.transformMixin = transformMixin;
//# sourceMappingURL=transform.mixin.js.map